@@ -33,7 +33,7 @@
 #ifndef PLR_H
 #define PLR_H
 
-#define PLR_VERSION		"08.03.00.18"
+#define PLR_VERSION		"8.4"
 
 #include "postgres.h"
 
@@ -317,53 +317,73 @@ extern void R_RunExitFinalizers(void);
 	HeapTuple		tup; \
 	HeapTupleHeader	dnewtup; \
 	HeapTupleHeader	dtrigtup
+
+
+#if (PG_VERSION_NUM >= 120000)
+
+#define SET_ARG(val, null, index) \
+    do { \
+	args[index].value=val; \
+	args[index].isnull =null; \
+    } while (0)
+
+#define IS_ARG_NULL(i) args[i].isnull
+#define GET_ARG_VALUE(i) args[i].value
+
+#else
+
+#define SET_ARG(val, null, index) \
+	do { \
+	arg[index]=val; \
+	argnull[index]=null; \
+	} while (0)
+
+#define IS_ARG_NULL(i) argnull[i]
+#define GET_ARG_VALUE(i) arg[i]
+
+#endif
+
 #define SET_INSERT_ARGS_567 \
 	do { \
-		arg[5] = DirectFunctionCall1(textin, CStringGetDatum("INSERT")); \
+		SET_ARG(DirectFunctionCall1(textin, CStringGetDatum("INSERT")),false,5); \
 		tup = trigdata->tg_trigtuple; \
 		dtrigtup = (HeapTupleHeader) palloc(tup->t_len); \
 		memcpy((char *) dtrigtup, (char *) tup->t_data, tup->t_len); \
 		HeapTupleHeaderSetDatumLength(dtrigtup, tup->t_len); \
 		HeapTupleHeaderSetTypeId(dtrigtup, tupdesc->tdtypeid); \
 		HeapTupleHeaderSetTypMod(dtrigtup, tupdesc->tdtypmod); \
-		arg[6] = PointerGetDatum(dtrigtup); \
-		argnull[6] = false; \
-		arg[7] = (Datum) 0; \
-		argnull[7] = true; \
+		SET_ARG(PointerGetDatum(dtrigtup),false,6); \
+		SET_ARG((Datum)0,true,7); \
 	} while (0)
 #define SET_DELETE_ARGS_567 \
 	do { \
-		arg[5] = DirectFunctionCall1(textin, CStringGetDatum("DELETE")); \
-		arg[6] = (Datum) 0; \
-		argnull[6] = true; \
+		SET_ARG(DirectFunctionCall1(textin, CStringGetDatum("DELETE")),false,5); \
+		SET_ARG((Datum) 0,true,6); \
 		tup = trigdata->tg_trigtuple; \
 		dtrigtup = (HeapTupleHeader) palloc(tup->t_len); \
 		memcpy((char *) dtrigtup, (char *) tup->t_data, tup->t_len); \
 		HeapTupleHeaderSetDatumLength(dtrigtup, tup->t_len); \
 		HeapTupleHeaderSetTypeId(dtrigtup, tupdesc->tdtypeid); \
 		HeapTupleHeaderSetTypMod(dtrigtup, tupdesc->tdtypmod); \
-		arg[7] = PointerGetDatum(dtrigtup); \
-		argnull[7] = false; \
+		SET_ARG(PointerGetDatum(dtrigtup),false,7); \
 	} while (0)
 #define SET_UPDATE_ARGS_567 \
 	do { \
-		arg[5] = DirectFunctionCall1(textin, CStringGetDatum("UPDATE")); \
+		SET_ARG(DirectFunctionCall1(textin, CStringGetDatum("UPDATE")),false,5); \
 		tup = trigdata->tg_newtuple; \
 		dnewtup = (HeapTupleHeader) palloc(tup->t_len); \
 		memcpy((char *) dnewtup, (char *) tup->t_data, tup->t_len); \
 		HeapTupleHeaderSetDatumLength(dnewtup, tup->t_len); \
 		HeapTupleHeaderSetTypeId(dnewtup, tupdesc->tdtypeid); \
 		HeapTupleHeaderSetTypMod(dnewtup, tupdesc->tdtypmod); \
-		arg[6] = PointerGetDatum(dnewtup); \
-		argnull[6] = false; \
+		SET_ARG(PointerGetDatum(dnewtup),false,6); \
 		tup = trigdata->tg_trigtuple; \
 		dtrigtup = (HeapTupleHeader) palloc(tup->t_len); \
 		memcpy((char *) dtrigtup, (char *) tup->t_data, tup->t_len); \
 		HeapTupleHeaderSetDatumLength(dtrigtup, tup->t_len); \
 		HeapTupleHeaderSetTypeId(dtrigtup, tupdesc->tdtypeid); \
 		HeapTupleHeaderSetTypMod(dtrigtup, tupdesc->tdtypmod); \
-		arg[7] = PointerGetDatum(dtrigtup); \
-		argnull[7] = false; \
+		SET_ARG(PointerGetDatum(dtrigtup),false,7); \
 	} while (0)
 #define CONVERT_TUPLE_TO_DATAFRAME \
 	do { \
@@ -371,7 +391,7 @@ extern void R_RunExitFinalizers(void);
 		int32		tupTypmod; \
 		TupleDesc	tupdesc; \
 		HeapTuple	tuple = palloc(sizeof(HeapTupleData)); \
-		HeapTupleHeader	tuple_hdr = DatumGetHeapTupleHeader(arg[i]); \
+		HeapTupleHeader	tuple_hdr = DatumGetHeapTupleHeader(GET_ARG_VALUE(i)); \
 		tupType = HeapTupleHeaderGetTypeId(tuple_hdr); \
 		tupTypmod = HeapTupleHeaderGetTypMod(tuple_hdr); \
 		tupdesc = lookup_rowtype_tupdesc(tupType, tupTypmod); \
@@ -386,16 +406,6 @@ extern void R_RunExitFinalizers(void);
 #define GET_ARG_NAMES \
 		char  **argnames; \
 		argnames = fetchArgNames(procTup, procStruct->pronargs)
-#define SET_ARG_NAME \
-	do { \
-		if (argnames && argnames[i] && argnames[i][0]) \
-		{ \
-			appendStringInfo(proc_internal_args, "%s", argnames[i]); \
-			pfree(argnames[i]); \
-		} \
-		else \
-			appendStringInfo(proc_internal_args, "arg%d", i + 1); \
-	} while (0)
 #define SET_FRAME_ARG_NAME \
 	do { \
 		appendStringInfo(proc_internal_args, "farg%d", i + 1); \
@@ -461,18 +471,11 @@ typedef struct plr_function
 	ItemPointerData		fn_tid;
 	plr_func_hashkey   *fn_hashkey; /* back-link to hashtable key */
 	bool				lanpltrusted;
-	Oid					result_typid;
-	bool				result_istuple;
-	FmgrInfo			result_in_func;
-	Oid					result_elem;
-	FmgrInfo			result_elem_in_func;
-	int					result_elem_typlen;
-	bool				result_elem_typbyval;
-	char				result_elem_typalign;
 	int					result_natts;
+	Oid				   *result_fld_typid;
 	Oid				   *result_fld_elem_typid;
 	FmgrInfo		   *result_fld_elem_in_func;
-	int				   *result_fld_elem_typlen;
+	int16			   *result_fld_elem_typlen;
 	bool			   *result_fld_elem_typbyval;
 	char			   *result_fld_elem_typalign;
 	int					nargs;
@@ -502,53 +505,53 @@ typedef struct plr_hashent
  * external declarations
  */
 
-/* libR interpreter initialization */
-extern int Rf_initEmbeddedR(int argc, char **argv);
-
 /* PL/R language handler */
-extern Datum plr_call_handler(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_call_handler(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_inline_handler(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_validator(PG_FUNCTION_ARGS);
 extern void PLR_CLEANUP;
 extern void plr_init(void);
 extern void plr_load_modules(void);
 extern void load_r_cmd(const char *cmd);
-extern SEXP call_r_func(SEXP fun, SEXP rargs);
+extern SEXP call_r_func(SEXP fun, SEXP rargs, SEXP rho);
 
 /* argument and return value conversion functions */
 extern SEXP pg_scalar_get_r(Datum dvalue, Oid arg_typid, FmgrInfo arg_out_func);
 extern SEXP pg_array_get_r(Datum dvalue, FmgrInfo out_func, int typlen, bool typbyval, char typalign);
-extern SEXP pg_datum_array_get_r(Datum *elem_values, bool *elem_nulls, int numels, bool has_nulls,
-								 Oid element_type, FmgrInfo out_func, bool typbyval);
+#ifdef HAVE_WINDOW_FUNCTIONS
+extern SEXP pg_window_frame_get_r(WindowObject winobj, int argno, plr_function* function);
+#endif
 extern SEXP pg_tuple_get_r_frame(int ntuples, HeapTuple *tuples, TupleDesc tupdesc);
 extern Datum r_get_pg(SEXP rval, plr_function *function, FunctionCallInfo fcinfo);
 extern Datum get_datum(SEXP rval, Oid typid, Oid typelem, FmgrInfo in_func, bool *isnull);
 extern Datum get_scalar_datum(SEXP rval, Oid result_typ, FmgrInfo result_in_func, bool *isnull);
 
 /* Postgres support functions installed into the R interpreter */
-extern void throw_pg_notice(const char **msg);
-extern SEXP plr_quote_literal(SEXP rawstr);
-extern SEXP plr_quote_ident(SEXP rawstr);
-extern SEXP plr_SPI_exec(SEXP rsql);
-extern SEXP plr_SPI_prepare(SEXP rsql, SEXP rargtypes);
-extern SEXP plr_SPI_execp(SEXP rsaved_plan, SEXP rargvalues);
-extern SEXP plr_SPI_cursor_open(SEXP cursor_name_arg,SEXP rsaved_plan, SEXP rargvalues);
-extern SEXP plr_SPI_cursor_fetch(SEXP cursor_in,SEXP forward_in, SEXP rows_in);
-extern void plr_SPI_cursor_close(SEXP cursor_in);
-extern void plr_SPI_cursor_move(SEXP cursor_in, SEXP forward_in, SEXP rows_in);
-extern SEXP plr_SPI_lastoid(void);
-extern void throw_r_error(const char **msg);
+PGDLLEXPORT void throw_pg_log(int* elevel, const char **msg);
+PGDLLEXPORT SEXP plr_quote_literal(SEXP rawstr);
+PGDLLEXPORT SEXP plr_quote_ident(SEXP rawstr);
+PGDLLEXPORT SEXP plr_SPI_exec(SEXP rsql);
+PGDLLEXPORT SEXP plr_SPI_prepare(SEXP rsql, SEXP rargtypes);
+PGDLLEXPORT SEXP plr_SPI_execp(SEXP rsaved_plan, SEXP rargvalues);
+PGDLLEXPORT SEXP plr_SPI_cursor_open(SEXP cursor_name_arg,SEXP rsaved_plan, SEXP rargvalues);
+PGDLLEXPORT SEXP plr_SPI_cursor_fetch(SEXP cursor_in,SEXP forward_in, SEXP rows_in);
+PGDLLEXPORT void plr_SPI_cursor_close(SEXP cursor_in);
+PGDLLEXPORT void plr_SPI_cursor_move(SEXP cursor_in, SEXP forward_in, SEXP rows_in);
+PGDLLEXPORT SEXP plr_SPI_lastoid(void);
+PGDLLEXPORT void throw_r_error(const char **msg);
 
 /* Postgres callable functions useful in conjunction with PL/R */
-extern Datum plr_version(PG_FUNCTION_ARGS);
-extern Datum reload_plr_modules(PG_FUNCTION_ARGS);
-extern Datum install_rcmd(PG_FUNCTION_ARGS);
-extern Datum plr_array_push(PG_FUNCTION_ARGS);
-extern Datum plr_array(PG_FUNCTION_ARGS);
-extern Datum plr_array_accum(PG_FUNCTION_ARGS);
-extern Datum plr_environ(PG_FUNCTION_ARGS);
-extern Datum plr_set_rhome(PG_FUNCTION_ARGS);
-extern Datum plr_unset_rhome(PG_FUNCTION_ARGS);
-extern Datum plr_set_display(PG_FUNCTION_ARGS);
-extern Datum plr_get_raw(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_version(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum reload_plr_modules(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum install_rcmd(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_array_push(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_array(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_array_accum(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_environ(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_set_rhome(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_unset_rhome(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_set_display(PG_FUNCTION_ARGS);
+PGDLLEXPORT Datum plr_get_raw(PG_FUNCTION_ARGS);
 
 /* Postgres backend support functions */
 extern void compute_function_hashkey(FunctionCallInfo fcinfo,
@@ -559,7 +562,7 @@ extern plr_function *plr_HashTableLookup(plr_func_hashkey *func_key);
 extern void plr_HashTableInsert(plr_function *function,
 								plr_func_hashkey *func_key);
 extern void plr_HashTableDelete(plr_function *function);
-extern char *get_load_self_ref_cmd(Oid funcid);
+extern char *get_load_self_ref_cmd(Oid langOid);
 extern void perm_fmgr_info(Oid functionId, FmgrInfo *finfo);
 
 #endif   /* PLR_H */
